package com.game_analytics

import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodChannel.Result
import android.app.Activity
import android.content.Context
import com.gameanalytics.sdk.*;
import org.json.JSONObject
import java.util.*

class GameAnalyticsPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    private lateinit var activity: Activity
    private lateinit var context: Context
    private lateinit var channel: MethodChannel

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "setBaseUrl" -> {
                setBaseUrl(call, result)
            }
            "configureAvailableCustomDimensions01" -> {
                configureAvailableCustomDimensions01(call, result)
            }
            "configureAvailableCustomDimensions02" -> {
                configureAvailableCustomDimensions02(call, result)
            }
            "configureAvailableCustomDimensions03" -> {
                configureAvailableCustomDimensions03(call, result)
            }
            "configureAvailableResourceCurrencies" -> {
                configureAvailableResourceCurrencies(call, result)
            }
            "configureAvailableResourceItemTypes" -> {
                configureAvailableResourceItemTypes(call, result)
            }
            "configureBuild" -> {
                configureBuild(call, result)
            }
            "configureSdkGameEngineVersion" -> {
                configureSdkGameEngineVersion(call, result)
            }
            "configureGameEngineVersion" -> {
                configureGameEngineVersion(call, result)
            }
            "configureAutoDetectAppVersion" -> {
                configureAutoDetectAppVersion(call, result)
            }
            "configureUserId" -> {
                configureUserId(call, result)
            }
            "initialize" -> {
                initialize(call, result)
            }
            "addBusinessEvent" -> {
                addBusinessEvent(call, result)
            }
            "addResourceEvent" -> {
                addResourceEvent(call, result)
            }
            "addProgressionEvent" -> {
                addProgressionEvent(call, result)
            }
            "addDesignEvent" -> {
                addDesignEvent(call, result)
            }
            "addErrorEvent" -> {
                addErrorEvent(call, result)
            }
            "addAdEvent" -> {
                addAdEvent(call, result)
            }
            "addImpressionMoPubEvent" -> {
                addImpressionMoPubEvent(call, result)
            }
            "addImpressionFyberEvent" -> {
                addImpressionFyberEvent(call, result)
            }
            "addImpressionIronSourceEvent" -> {
                addImpressionIronSourceEvent(call, result)
            }
            "addImpressionTopOnEvent" -> {
                addImpressionTopOnEvent(call, result)
            }
            "addImpressionHyperBidEvent" -> {
                addImpressionHyperBidEvent(call, result)
            }
            "addImpressionMaxEvent" -> {
                addImpressionMaxEvent(call, result)
            }
            "addImpressionAequusEvent" -> {
                addImpressionAequusEvent(call, result)
            }
            "addImpressionEvent" -> {
                addImpressionEvent(call, result)
            }
            "setEnabledInfoLog" -> {
                setEnabledInfoLog(call, result)
            }
            "setEnabledVerboseLog" -> {
                setEnabledVerboseLog(call, result)
            }
            "setEnabledManualSessionHandling" -> {
                setEnabledManualSessionHandling(call, result)
            }
            "setEnabledErrorReporting" -> {
                setEnabledErrorReporting(call, result)
            }
            "setEnabledEventSubmission" -> {
                setEnabledEventSubmission(call, result)
            }
            "setCustomDimension01" -> {
                setCustomDimension01(call, result)
            }
            "setCustomDimension02" -> {
                setCustomDimension02(call, result)
            }
            "setCustomDimension03" -> {
                setCustomDimension03(call, result)
            }
            "startTimer" -> {
                startTimer(call, result)
            }
            "pauseTimer" -> {
                pauseTimer(call, result)
            }
            "resumeTimer" -> {
                resumeTimer(call, result)
            }
            "stopTimer" -> {
                stopTimer(call, result)
            }
            "getRemoteConfigsValueAsString" -> {
                getRemoteConfigsValueAsString(call, result)
            }
            "isRemoteConfigsReady" -> {
                isRemoteConfigsReady(call, result)
            }
            "getRemoteConfigsContentAsString" -> {
                getRemoteConfigsContentAsString(call, result)
            }
            "getABTestingId" -> {
                getABTestingId(call, result)
            }
            "getABTestingVariantId" -> {
                getABTestingVariantId(call, result)
            }
            "startSession" -> {
                startSession(call, result)
            }
            "endSession" -> {
                endSession(call, result)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    fun setBaseUrl(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val url = args["url"] as String
        GameAnalytics.setBaseUrl(url)
        result.success(null)
    }

    fun configureAvailableCustomDimensions01(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val customDimensions = args["customDimensions"] as List<String>
        GameAnalytics.configureAvailableCustomDimensions01(customDimensions.toString())
        result.success(null)
    }

    fun configureAvailableCustomDimensions02(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val customDimensions = args["customDimensions"] as List<String>
        GameAnalytics.configureAvailableCustomDimensions02(customDimensions.toString())
        result.success(null)
    }

    fun configureAvailableCustomDimensions03(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val customDimensions = args["customDimensions"] as List<String>
        GameAnalytics.configureAvailableCustomDimensions03(customDimensions.toString())
        result.success(null)
    }

    fun configureAvailableResourceCurrencies(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val resourceCurrencies = args["resourceCurrencies"] as List<String>
        GameAnalytics.configureAvailableResourceCurrencies(resourceCurrencies.toString())
        result.success(null)
    }

    fun configureAvailableResourceItemTypes(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val resourceItemTypes = args["resourceItemTypes"] as List<String>
        GameAnalytics.configureAvailableResourceItemTypes(resourceItemTypes.toString())
        result.success(null)
    }

    fun configureBuild(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val build = args["build"] as String
        GameAnalytics.configureBuild(build)
        result.success(null)
    }

    fun configureSdkGameEngineVersion(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val sdkGameEngineVersion = args["sdkGameEngineVersion"] as String
        GameAnalytics.configureSdkGameEngineVersion(sdkGameEngineVersion)
        result.success(null)
    }

    fun configureGameEngineVersion(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val gameEngineVersion = args["gameEngineVersion"] as String
        GameAnalytics.configureGameEngineVersion(gameEngineVersion)
        result.success(null)
    }

    fun configureAutoDetectAppVersion(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val flag = args["flag"] as Boolean
        GameAnalytics.configureAutoDetectAppVersion(flag)
        result.success(null)
    }

    fun configureUserId(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val uId = args["uId"] as String
        GameAnalytics.configureUserId(uId)
        result.success(null)
    }

    fun initialize(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val gameKey = args["gameKey"] as String
        val gameSecret = args["gameSecret"] as String
        GameAnalytics.initialize(activity, gameKey, gameSecret)
        result.success(null)
    }

    fun addBusinessEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val currency = args["currency"] as String
        val amount = args["amount"] as Int
        val itemType = args["itemType"] as String
        val itemId = args["itemId"] as String
        val cartType = args["cartType"] as String
        val receipt = args["receipt"] as String
        val store = args["store"] as String
        val signature = args["signature"] as String
        GameAnalytics.addBusinessEvent(currency, amount, itemType, itemId, cartType, receipt, store, signature)
        result.success(null)
    }

    fun addResourceEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val flowType = args["flowType"] as Int
        val currency = args["currency"] as String
        val amount = args["amount"] as Float
        val itemType = args["itemType"] as String
        val itemId = args["itemId"] as String
        GameAnalytics.addResourceEvent(flowType, currency, amount, itemType, itemId)
        result.success(null)
    }

    fun addProgressionEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val progressionStatus = args["progressionStatus"] as Int
        val progression01 = args["progression01"] as String
        val progression02 = args["progression02"] as String
        val progression03 = args["progression03"] as String
        val score = args["score"] as Double
        GameAnalytics.addProgressionEvent(progressionStatus, progression01, progression02, progression03, score)
        result.success(null)
    }

    fun addDesignEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val eventId = args["eventId"] as String
        val value = args["value"] as Double
        GameAnalytics.addDesignEvent(eventId, value)
        result.success(null)
    }

    fun addErrorEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val severity = args["severity"] as Int
        val message = args["message"] as String
        GameAnalytics.addErrorEvent(severity, message)
        result.success(null)
    }

    fun addAdEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adAction = args["adAction"] as Int
        val adType = args["adType"] as Int
        val adSdkName = args["adSdkName"] as String
        val adPlacement = args["adPlacement"] as String
        val noAdReason = args["noAdReason"] as Int?
        val duration = args["duration"] as Long?
        when {
            noAdReason != null -> {
                GameAnalytics.addAdEvent(adAction, adType, adSdkName, adPlacement, noAdReason)
            }
            duration != null -> {
                GameAnalytics.addAdEvent(adAction, adType, adSdkName, adPlacement, duration)
            }
            else -> {
                GameAnalytics.addAdEvent(adAction, adType, adSdkName, adPlacement)
            }
        }
        result.success(null)
    }

    fun addImpressionMoPubEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adNetworkVersion = args["adNetworkVersion"] as String
        val impressionData = JSONObject(args["impressionData"] as Map<String, *>)
        GameAnalytics.addImpressionMoPubEvent(adNetworkVersion, impressionData)
        result.success(null)
    }

    fun addImpressionFyberEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adNetworkVersion = args["adNetworkVersion"] as String
        val impressionData = JSONObject(args["impressionData"] as Map<String, *>)
        GameAnalytics.addImpressionFyberEvent(adNetworkVersion, impressionData)
        result.success(null)
    }

    fun addImpressionIronSourceEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adNetworkVersion = args["adNetworkVersion"] as String
        val impressionData = JSONObject(args["impressionData"] as Map<String, *>)
        GameAnalytics.addImpressionIronSourceEvent(adNetworkVersion, impressionData)
        result.success(null)
    }

    fun addImpressionTopOnEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adNetworkVersion = args["adNetworkVersion"] as String
        val impressionData = JSONObject(args["impressionData"] as Map<String, *>)
        GameAnalytics.addImpressionTopOnEvent(adNetworkVersion, impressionData)
        result.success(null)
    }

    fun addImpressionHyperBidEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adNetworkVersion = args["adNetworkVersion"] as String
        val impressionData = JSONObject(args["impressionData"] as Map<String, *>)
        GameAnalytics.addImpressionHyperBidEvent(adNetworkVersion, impressionData)
        result.success(null)
    }

    fun addImpressionMaxEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adNetworkVersion = args["adNetworkVersion"] as String
        val impressionData = JSONObject(args["impressionData"] as Map<String, *>)
        GameAnalytics.addImpressionMaxEvent(adNetworkVersion, impressionData)
        result.success(null)
    }

    fun addImpressionAequusEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adNetworkVersion = args["adNetworkVersion"] as String
        val impressionData = JSONObject(args["impressionData"] as Map<String, *>)
        GameAnalytics.addImpressionAequusEvent(adNetworkVersion, impressionData)
        result.success(null)
    }

    fun addImpressionEvent(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val adNetworkName = args["adNetworkName"] as String
        val adNetworkVersion = args["adNetworkVersion"] as String
        val impressionData = JSONObject(args["impressionData"] as Map<String, *>)
        GameAnalytics.addImpressionEvent(adNetworkName, adNetworkVersion, impressionData)
        result.success(null)
    }

    fun setEnabledInfoLog(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val flag = args["flag"] as Boolean
        GameAnalytics.setEnabledInfoLog(flag)
        result.success(null)
    }

    fun setEnabledVerboseLog(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val flag = args["flag"] as Boolean
        GameAnalytics.setEnabledVerboseLog(flag)
        result.success(null)
    }

    fun setEnabledManualSessionHandling(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val flag = args["flag"] as Boolean
        GameAnalytics.setEnabledManualSessionHandling(flag)
        result.success(null)
    }

    fun setEnabledErrorReporting(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val flag = args["flag"] as Boolean
        GameAnalytics.setEnabledErrorReporting(flag)
        result.success(null)
    }

    fun setEnabledEventSubmission(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val flag = args["flag"] as Boolean
        GameAnalytics.setEnabledEventSubmission(flag)
        result.success(null)
    }

    fun setCustomDimension01(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val dimension = args["dimension"] as String
        GameAnalytics.setCustomDimension01(dimension)
        result.success(null)
    }

    fun setCustomDimension02(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val dimension = args["dimension"] as String
        GameAnalytics.setCustomDimension02(dimension)
        result.success(null)
    }

    fun setCustomDimension03(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val dimension = args["dimension"] as String
        GameAnalytics.setCustomDimension03(dimension)
        result.success(null)
    }

    fun startTimer(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val key = args["key"] as String
        GameAnalytics.startTimer(key)
        result.success(null)
    }

    fun pauseTimer(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val key = args["key"] as String
        GameAnalytics.pauseTimer(key)
        result.success(null)
    }

    fun resumeTimer(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val key = args["key"] as String
        GameAnalytics.resumeTimer(key)
        result.success(null)
    }

    fun stopTimer(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val key = args["key"] as String
        GameAnalytics.stopTimer(key)
        result.success(null)
    }

    fun getRemoteConfigsValueAsString(call: MethodCall, result: Result) {
        val args = call.arguments as Map<*, *>
        val key = args["key"] as String
        val defaultValue = args["defaultValue"] as String
        result.success(GameAnalytics.getRemoteConfigsValueAsString(key, defaultValue))
    }

    fun isRemoteConfigsReady(call: MethodCall, result: Result) {
        result.success(GameAnalytics.isRemoteConfigsReady())
    }

    fun getRemoteConfigsContentAsString(call: MethodCall, result: Result) {
        result.success(GameAnalytics.getRemoteConfigsContentAsString())
    }

    fun getABTestingId(call: MethodCall, result: Result) {
        result.success(GameAnalytics.getABTestingId())
    }

    fun getABTestingVariantId(call: MethodCall, result: Result) {
        result.success(GameAnalytics.getABTestingVariantId())
    }

    fun startSession(call: MethodCall, result: Result) {
        GameAnalytics.startSession()
        result.success(null)
    }

    fun endSession(call: MethodCall, result: Result) {
        GameAnalytics.endSession()
        result.success(null)
    }

    //! Defaults

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "game_analytics")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
//        GameAnalytics.addRemoteConfigsListener(this);
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
//        GameAnalytics.removeRemoteConfigsListener(this);
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {}

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {}

    override fun onDetachedFromActivity() {}

//    override fun onRemoteConfigsUpdated() {
//        channel.invokeMethod("onRemoteConfigsUpdated", null)
//    }
}
